@@ -1,20 +1,37 @@
 // SPDX-License-Identifier: GPL-2.0
 
 #include "divefilter.h"
-#include "divelist.h" // for filter_dive
+#include "divelist.h"
 #include "gettextfromc.h"
 #include "qthelper.h"
+#include "selection.h"
 #include "subsurface-qt/divelistnotifier.h"
-#ifndef SUBSURFACE_MOBILE
+#if !defined(SUBSURFACE_MOBILE) && !defined(SUBSURFACE_DOWNLOADER)
 #include "desktop-widgets/mapwidget.h"
 #include "desktop-widgets/mainwindow.h"
 #include "desktop-widgets/divelistview.h"
 #include "qt-models/filtermodels.h"
 #endif
 
-static void updateDiveStatus(dive *d, bool newStatus, ShownChange &change)
+// Set filter status of dive and return whether it has been changed
+bool DiveFilter::setFilterStatus(struct dive *d, bool shown) const
 {
-	if (filter_dive(d, newStatus)) {
+	bool old_shown, changed;
+	if (!d)
+		return false;
+	old_shown = !d->hidden_by_filter;
+	d->hidden_by_filter = !shown;
+	if (!shown && d->selected)
+		deselect_dive(d);
+	changed = old_shown != shown;
+	if (changed)
+		shown_dives += shown - old_shown;
+	return changed;
+}
+
+void DiveFilter::updateDiveStatus(dive *d, bool newStatus, ShownChange &change) const
+{
+	if (setFilterStatus(d, newStatus)) {
 		if (newStatus)
 			change.newShown.push_back(d);
 		else
@@ -52,6 +69,16 @@ ShownChange DiveFilter::update(const QVector<dive *> &dives) const
 	return res;
 }
 
+void DiveFilter::reset()
+{
+	int i;
+	dive *d;
+	shown_dives = dive_table.nr;
+	for_each_dive(i, d)
+		d->hidden_by_filter = false;
+	updateAll();
+}
+
 ShownChange DiveFilter::updateAll() const
 {
 	dive *old_current = current_dive;
@@ -87,10 +114,18 @@ DiveFilter *DiveFilter::instance()
 	return &self;
 }
 
-DiveFilter::DiveFilter() : diveSiteRefCount(0)
+DiveFilter::DiveFilter() :
+	shown_dives(0),
+	diveSiteRefCount(0)
 {
 }
 
+void DiveFilter::diveRemoved(const dive *d) const
+{
+	if (!d->hidden_by_filter)
+		--shown_dives;
+}
+
 bool DiveFilter::showDive(const struct dive *d) const
 {
 	if (d->invalid && !prefs.display_invalid_dives)
@@ -103,7 +138,7 @@ bool DiveFilter::showDive(const struct dive *d) const
 			   [d] (const filter_constraint &c) { return filter_constraint_match_dive(c, d); });
 }
 
-#ifndef SUBSURFACE_MOBILE
+#if !defined(SUBSURFACE_MOBILE) && !defined(SUBSURFACE_DOWNLOADER)
 void DiveFilter::startFilterDiveSites(QVector<dive_site *> ds)
 {
 	if (++diveSiteRefCount > 1) {
@@ -166,8 +201,30 @@ QString DiveFilter::shownText() const
 		return gettextFromC::tr("%L1 dives").arg(dive_table.nr);
 }
 
+int DiveFilter::shownDives() const
+{
+	return shown_dives;
+}
+
 void DiveFilter::setFilter(const FilterData &data)
 {
 	filterData = data;
 	emit diveListNotifier.filterReset();
 }
+
+std::vector<dive *> DiveFilter::visibleDives() const
+{
+	if (shown_dives <= 0)
+		return {};
+
+	std::vector<dive *> res;
+	res.reserve(shown_dives);
+
+	int i;
+	dive *d;
+	for_each_dive(i, d) {
+		if (!d->hidden_by_filter)
+			res.push_back(d);
+	}
+	return res;
+}
