@@ -130,11 +130,22 @@ static int push_transfer_progress_cb(unsigned int current, unsigned int total, s
 	return git_storage_update_progress(buf);
 }
 
-char *get_local_dir(const char *remote, const char *branch)
+char *get_local_dir(const char *remote_in, const char *branch)
 {
 	SHA_CTX ctx;
 	unsigned char hash[20];
 
+	// this optimization could in theory lead to odd things happening if the
+	// cloud backend servers ever get out of sync - but when a user switches
+	// between those servers (either because one is down, or because the algorithm
+	// which server to pick changed, or because the user is on a different continent),
+	// then the hash and therefore the local directory would change. To prevent that
+	// from happening, normalize the cloud string to always use the old default name.
+	// That's trivial with QString operations and painful to do right in plain C, so
+	// let's be lazy and call a C++ helper function
+	// just remember to free the string we get back
+	const char *remote = normalize_cloud_name(remote_in);
+
 	// That zero-byte update is so that we don't get hash
 	// collisions for "repo1 branch" vs "repo 1branch".
 	SHA1_Init(&ctx);
@@ -142,7 +153,7 @@ char *get_local_dir(const char *remote, const char *branch)
 	SHA1_Update(&ctx, "", 1);
 	SHA1_Update(&ctx, branch, strlen(branch));
 	SHA1_Final(hash, &ctx);
-
+	free((void *)remote);
 	return format_string("%s/cloudstorage/%02x%02x%02x%02x%02x%02x%02x%02x",
 			system_default_directory(),
 			hash[0], hash[1], hash[2], hash[3],
@@ -297,7 +308,8 @@ int certificate_check_cb(git_cert *cert, int valid, const char *host, void *payl
 	UNUSED(payload);
 	if (verbose)
 		SSRF_INFO("git storage: certificate callback for host %s with validity %d\n", host, valid);
-	if (same_string(host, "cloud.subsurface-divelog.org") && cert->cert_type == GIT_CERT_X509) {
+	if ((same_string(host, CLOUD_HOST_GENERIC) || same_string(host, CLOUD_HOST_US) || same_string(host, CLOUD_HOST_EU)) &&
+			cert->cert_type == GIT_CERT_X509) {
 		// for some reason the LetsEncrypt certificate makes libgit2 throw up on some
 		// platforms but not on others
 		// if we are connecting to the cloud server we alrady called 'canReachCloudServer()'
@@ -704,15 +716,18 @@ int sync_with_remote(git_repository *repo, const char *remote, const char *branc
 		return 0;
 	}
 
-	if (is_subsurface_cloud && !canReachCloudServer()) {
+	// we know that we already checked for the cloud server, but to give a decent warning message
+	// here in case none of them are reachable, let's check one more time
+	if (is_subsurface_cloud && !canReachCloudServer(&remote)) {
 		// this is not an error, just a warning message, so return 0
 		SSRF_INFO("git storage: cannot connect to remote server");
 		report_error("Cannot connect to cloud server, working with local copy");
 		git_storage_update_progress(translate("gettextFromC", "Can't reach cloud server, working with local data"));
 		return 0;
 	}
+
 	if (verbose)
-		SSRF_INFO("git storage: fetch remote\n");
+		SSRF_INFO("git storage: fetch remote %s\n", git_remote_url(origin));
 	git_fetch_options opts = GIT_FETCH_OPTIONS_INIT;
 	opts.callbacks.transfer_progress = &transfer_progress_cb;
 	auth_attempt = 0;
@@ -775,6 +790,11 @@ static git_repository *update_local_repo(const char *localdir, const char *remot
 		}
 		git_reference_free(head);
 	}
+	/* make sure we have the correct origin - the cloud server URL could have changed */
+	if (git_remote_set_url(repo, "origin", remote)) {
+		SSRF_INFO("git storage: failed to update origin to '%s'", remote);
+		return NULL;
+	}
 
 	if (!git_local_only)
 		sync_with_remote(repo, remote, branch, rt);
@@ -869,7 +889,7 @@ static git_repository *create_local_repo(const char *localdir, const char *remot
 	opts.fetch_opts.callbacks.certificate_check = certificate_check_cb;
 
 	opts.checkout_branch = branch;
-	if (is_subsurface_cloud && !canReachCloudServer()) {
+	if (is_subsurface_cloud && !canReachCloudServer(&remote)) {
 		SSRF_INFO("git storage: cannot reach remote server");
 		return 0;
 	}
@@ -965,10 +985,10 @@ static struct git_repository *get_remote_repo(const char *localdir, const char *
  *    https://host/repo[branch]
  *    file://repo[branch]
  */
-static struct git_repository *is_remote_git_repository(char *remote, const char *branch)
+static struct git_repository *is_remote_git_repository(char **remote, const char *branch)
 {
 	char c, *localdir;
-	char *p = remote;
+	char *p = *remote;
 
 	while ((c = *p++) >= 'a' && c <= 'z')
 		/* nothing */;
@@ -1004,7 +1024,7 @@ static struct git_repository *is_remote_git_repository(char *remote, const char
 	 * next we need to make sure that any encoded username
 	 * has been extracted from the URL
 	 */
-	char *at = strchr(remote, '@');
+	char *at = strchr(*remote, '@');
 	if (at) {
 		/* was this the @ that denotes an account? that means it was before the
 		 * first '/' after the protocol:// - so let's find a '/' after that and compare */
@@ -1017,15 +1037,23 @@ static struct git_repository *is_remote_git_repository(char *remote, const char
 			memmove(p, at + 1, strlen(at + 1) + 1);
 		}
 	}
-	localdir = get_local_dir(remote, branch);
+	localdir = get_local_dir(*remote, branch);
 	if (!localdir)
 		return NULL;
 
 	/* remember if the current git storage we are working on is our cloud storage
 	 * this is used to create more user friendly error message and warnings */
-	is_subsurface_cloud = strstr(remote, prefs.cloud_git_url) != NULL;
+	is_subsurface_cloud = strstr(*remote, prefs.cloud_base_url) != NULL;
 
-	return get_remote_repo(localdir, remote, branch);
+	/* if we are planning to access the server, make sure it's available and try to
+	 * pick one of the alternative servers if necessary */
+	if (is_subsurface_cloud && !git_local_only) {
+		// since we know that this is Subsurface cloud storage, we don't have to
+		// worry about the local directory name changing if we end up with a different
+		// cloud_base_url... the algorithm normalizes those URLs
+		(void)canReachCloudServer((const char **)remote);
+	}
+	return get_remote_repo(localdir, *remote, branch);
 }
 
 /*
@@ -1099,7 +1127,7 @@ struct git_repository *is_git_repository(const char *filename, const char **bran
 		*remote = loc;
 		return dummy_git_repository;
 	}
-	repo = is_remote_git_repository(loc, branch);
+	repo = is_remote_git_repository(&loc, branch);
 	if (repo) {
 		if (remote)
 			*remote = loc;
