@@ -4,9 +4,16 @@
 
 #define MAX_EVENT_NAME 128
 
-#include "dive.h" // for struct event!
+#include "event.h"
+#include "equipment.h" // for cylinder_t
+#include "extradata.h"
+#include "filterpreset.h"
+#include "picture.h"
 
 #include <sqlite3.h>
+#include <time.h>
+
+struct xml_params;
 
 typedef union {
 	struct event event;
@@ -51,20 +58,32 @@ struct parser_state {
 	struct dive_trip *cur_trip;		/* owning */
 	struct sample *cur_sample;		/* non-owning */
 	struct picture cur_picture;		/* owning */
+	struct filter_preset *cur_filter;	/* owning */
+	char *fulltext;				/* owning */
+	char *fulltext_string_mode;		/* owning */
+	char *filter_constraint_type;		/* owning */
+	char *filter_constraint_string_mode;	/* owning */
+	char *filter_constraint_range_mode;	/* owning */
+	bool filter_constraint_negate;
+	char *filter_constraint;		/* owning */
 	char *country, *city;			/* owning */
 	int taxonomy_category, taxonomy_origin;
 
 	bool in_settings;
 	bool in_userid;
+	bool in_fulltext;
+	bool in_filter_constraint;
 	struct tm cur_tm;
 	int lastcylinderindex, next_o2_sensor;
 	int o2pressure_sensor;
 	int sample_rate;
 	struct extra_data cur_extra_data;
 	struct units xml_parsing_units;
-	struct dive_table *target_table;	/* non-owning */
-	struct trip_table *trips;		/* non-owning */
-	struct dive_site_table *sites;		/* non-owning */
+	struct dive_table *target_table;		/* non-owning */
+	struct trip_table *trips;			/* non-owning */
+	struct dive_site_table *sites;			/* non-owning */
+	struct device_table *devices;			/* non-owning */
+	struct filter_preset_table *filter_presets;	/* non-owning */
 
 	sqlite3 *sql_handle;			/* for SQL based parsers */
 	event_allocation_t event_allocation;
@@ -100,6 +119,12 @@ void dive_site_start(struct parser_state *state);
 void dive_site_end(struct parser_state *state);
 void dive_start(struct parser_state *state);
 void dive_end(struct parser_state *state);
+void filter_preset_start(struct parser_state *state);
+void filter_preset_end(struct parser_state *state);
+void filter_constraint_start(struct parser_state *state);
+void filter_constraint_end(struct parser_state *state);
+void fulltext_start(struct parser_state *state);
+void fulltext_end(struct parser_state *state);
 void trip_start(struct parser_state *state);
 void trip_end(struct parser_state *state);
 void picture_start(struct parser_state *state);
@@ -120,14 +145,26 @@ void utf8_string(char *buffer, void *_res);
 void add_dive_site(char *ds_name, struct dive *dive, struct parser_state *state);
 int atoi_n(char *ptr, unsigned int len);
 
-int parse_dm4_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips, struct dive_site_table *sites);
-int parse_dm5_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips, struct dive_site_table *sites);
-int parse_seac_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips, struct dive_site_table *sites);
-int parse_shearwater_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips, struct dive_site_table *sites);
-int parse_shearwater_cloud_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips, struct dive_site_table *sites);
-int parse_cobalt_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips, struct dive_site_table *sites);
-int parse_divinglog_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips, struct dive_site_table *sites);
-int parse_dlf_buffer(unsigned char *buffer, size_t size, struct dive_table *table, struct trip_table *trips, struct dive_site_table *sites);
+void parse_xml_init(void);
+int parse_xml_buffer(const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips, struct dive_site_table *sites,
+		     struct device_table *devices, struct filter_preset_table *filter_presets, const struct xml_params *params);
+void parse_xml_exit(void);
+int parse_dm4_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips,
+		     struct dive_site_table *sites, struct device_table *devices);
+int parse_dm5_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips,
+		     struct dive_site_table *sites, struct device_table *devices);
+int parse_seac_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips,
+		      struct dive_site_table *sites, struct device_table *devices);
+int parse_shearwater_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips,
+			    struct dive_site_table *sites, struct device_table *devices);
+int parse_shearwater_cloud_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips,
+				  struct dive_site_table *sites, struct device_table *devices);
+int parse_cobalt_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips,
+			struct dive_site_table *sites, struct device_table *devices);
+int parse_divinglog_buffer(sqlite3 *handle, const char *url, const char *buf, int size, struct dive_table *table, struct trip_table *trips,
+			   struct dive_site_table *sites, struct device_table *devices);
+int parse_dlf_buffer(unsigned char *buffer, size_t size, struct dive_table *table, struct trip_table *trips,
+		     struct dive_site_table *sites, struct device_table *devices);
 #ifdef __cplusplus
 }
 #endif
