@@ -22,6 +22,7 @@
 #include "trip.h"
 #include "imagedownloader.h"
 #include "xmlparams.h"
+#include "core/git-access.h" // for CLOUD_HOST definitions
 #include <QFile>
 #include <QRegExp>
 #include <QDir>
@@ -1442,9 +1443,9 @@ int getCloudURL(QString &filename)
 		free((void *)prefs.cloud_storage_email_encoded);
 		prefs.cloud_storage_email_encoded = copy_qstring(email);
 	}
-	filename = QString(QString(prefs.cloud_git_url) + "/%1[%1]").arg(email);
+	filename = QString(QString(prefs.cloud_base_url) + "git/%1[%1]").arg(email);
 	if (verbose)
-		qDebug() << "cloud URL set as" << filename;
+		qDebug() << "returning cloud URL" << filename;
 	return 0;
 }
 
@@ -1455,6 +1456,15 @@ extern "C" char *cloud_url()
 	return copy_qstring(filename);
 }
 
+extern "C" const char *normalize_cloud_name(const char *remote_in)
+{
+	// replace ssrf-cloud-XX.subsurface... names with cloud.subsurface... names
+	// that trailing '/' is to match old code
+	QString ri(remote_in);
+	ri.replace(QRegularExpression(CLOUD_HOST_PATTERN), CLOUD_HOST_GENERIC "/");
+	return strdup(ri.toUtf8().constData());
+}
+
 extern "C" bool getProxyString(char **buffer)
 {
 	if (prefs.proxy_type == QNetworkProxy::HttpProxy) {
