@@ -8,11 +8,10 @@
 
 class qPrefLocationService : public QObject {
 	Q_OBJECT
-	Q_PROPERTY(int distance_threshold READ distance_threshold WRITE set_distance_threshold NOTIFY distance_thresholdChanged);
-	Q_PROPERTY(int time_threshold READ time_threshold WRITE set_time_threshold NOTIFY time_thresholdChanged);
+	Q_PROPERTY(int distance_threshold READ distance_threshold WRITE set_distance_threshold NOTIFY distance_thresholdChanged)
+	Q_PROPERTY(int time_threshold READ time_threshold WRITE set_time_threshold NOTIFY time_thresholdChanged)
 
 public:
-	qPrefLocationService(QObject *parent = NULL);
 	static qPrefLocationService *instance();
 
 	// Load/Sync local settings (disk) and struct preference
@@ -34,6 +33,8 @@ signals:
 
 
 private:
+	qPrefLocationService() {}
+
 	static void disk_distance_threshold(bool doSync);
 	static void disk_time_threshold(bool doSync);
 };
