@@ -134,7 +134,7 @@ MainWindow::MainWindow() : QMainWindow(),
 	// for the "default" mode
 	mainTab.reset(new MainTab);
 	diveList.reset(new DiveListView);
-	graphics = new ProfileWidget2(this);
+	graphics = new ProfileWidget2(DivePlannerPointsModel::instance(), this);
 	mapWidget.reset(MapWidget::instance()); // Yes, this is ominous see comment in mapwidget.cpp.
 	plannerWidgets.reset(new PlannerWidgets);
 	statistics.reset(new StatsWidget);
@@ -223,6 +223,7 @@ MainWindow::MainWindow() : QMainWindow(),
 	enableDisableCloudActions();
 
 	ui.mainErrorMessage->hide();
+	setEnabledToolbar(false);
 	graphics->setEmptyState();
 	initialUiSetup();
 	readSettings();
@@ -295,10 +296,6 @@ MainWindow::MainWindow() : QMainWindow(),
 	connect(ui.profPn2, &QAction::triggered, pp_gas, &qPrefPartialPressureGas::set_pn2);
 	connect(ui.profPO2, &QAction::triggered, pp_gas, &qPrefPartialPressureGas::set_po2);
 
-	// now let's set up some connections
-	connect(graphics, &ProfileWidget2::enableToolbar ,this, &MainWindow::setEnabledToolbar);
-	connect(graphics, &ProfileWidget2::disableShortcuts, this, &MainWindow::disableShortcuts);
-	connect(graphics, &ProfileWidget2::enableShortcuts, this, &MainWindow::enableShortcuts);
 	connect(graphics, &ProfileWidget2::editCurrentDive, this, &MainWindow::editCurrentDive);
 
 	connect(&diveListNotifier, &DiveListNotifier::settingsChanged, graphics, &ProfileWidget2::settingsChanged);
@@ -427,6 +424,12 @@ void MainWindow::configureToolbar()
 	}
 }
 
+void MainWindow::plotCurrentDive()
+{
+	setEnabledToolbar(current_dive != nullptr);
+	graphics->plotDive(current_dive, dc_number);
+}
+
 void MainWindow::selectionChanged()
 {
 	mainTab->updateDiveInfo();
@@ -434,7 +437,7 @@ void MainWindow::selectionChanged()
 		configureToolbar();
 		enableDisableOtherDCsActions();
 	}
-	graphics->plotDive(current_dive, false);
+	plotCurrentDive();
 	MapWidget::instance()->selectionChanged();
 }
 
@@ -625,8 +628,8 @@ void MainWindow::closeCurrentFile()
 
 void MainWindow::updateCloudOnlineStatus()
 {
-	bool is_cloud = existing_filename && prefs.cloud_git_url && prefs.cloud_verification_status == qPrefCloudStorage::CS_VERIFIED &&
-			strstr(existing_filename, prefs.cloud_git_url);
+	bool is_cloud = existing_filename && prefs.cloud_verification_status == qPrefCloudStorage::CS_VERIFIED &&
+			strstr(existing_filename, prefs.cloud_base_url);
 	ui.actionCloudOnline->setEnabled(is_cloud);
 	ui.actionCloudOnline->setChecked(is_cloud && !git_local_only);
 }
@@ -685,7 +688,7 @@ void MainWindow::enableShortcuts()
 void MainWindow::showProfile()
 {
 	enableShortcuts();
-	graphics->setProfileState();
+	graphics->setProfileState(current_dive, dc_number);
 	setApplicationState(ApplicationState::Default);
 }
 
@@ -738,7 +741,7 @@ void MainWindow::refreshProfile()
 {
 	showProfile();
 	configureToolbar();
-	graphics->plotDive(current_dive, true);
+	plotCurrentDive();
 }
 
 void MainWindow::planCanceled()
@@ -771,6 +774,8 @@ void MainWindow::on_actionReplanDive_triggered()
 	// put us in PLAN mode
 	setApplicationState(ApplicationState::PlanDive);
 
+	disableShortcuts(true);
+	graphics->setPlanState(&displayed_dive, 0);
 	plannerWidgets->replanDive();
 }
 
@@ -782,6 +787,8 @@ void MainWindow::on_actionDivePlanner_triggered()
 	// put us in PLAN mode
 	setApplicationState(ApplicationState::PlanDive);
 
+	disableShortcuts(true);
+	graphics->setPlanState(&displayed_dive, 0);
 	plannerWidgets->planDive();
 }
 
@@ -919,7 +926,7 @@ void MainWindow::on_actionPreviousDC_triggered()
 	unsigned nrdc = number_of_computers(current_dive);
 	dc_number = (dc_number + nrdc - 1) % nrdc;
 	configureToolbar();
-	graphics->plotDive(current_dive, false);
+	plotCurrentDive();
 	mainTab->updateDiveInfo();
 }
 
@@ -928,7 +935,7 @@ void MainWindow::on_actionNextDC_triggered()
 	unsigned nrdc = number_of_computers(current_dive);
 	dc_number = (dc_number + 1) % nrdc;
 	configureToolbar();
-	graphics->plotDive(current_dive, false);
+	plotCurrentDive();
 	mainTab->updateDiveInfo();
 }
 
@@ -1167,7 +1174,7 @@ void MainWindow::loadRecentFiles()
 		QString file = s.value(key).toString();
 
 		// never add our cloud URL to the recent files
-		if (!same_string(prefs.cloud_git_url, "") && file.startsWith(prefs.cloud_git_url))
+		if (file.startsWith(prefs.cloud_base_url))
 			continue;
 		// but allow local git repos
 		QRegularExpression gitrepo("(.*)\\[[^]]+]");
@@ -1205,7 +1212,7 @@ void MainWindow::updateRecentFilesMenu()
 void MainWindow::addRecentFile(const QString &file, bool update)
 {
 	// never add Subsurface cloud file to the recent files - it has its own menu entry
-	if (!same_string(prefs.cloud_git_url, "") && file.startsWith(prefs.cloud_git_url))
+	if (file.startsWith(prefs.cloud_base_url))
 		return;
 	QString localFile = QDir::toNativeSeparators(file);
 	int index = recentFiles.indexOf(localFile);
@@ -1255,9 +1262,8 @@ int MainWindow::file_save_as(void)
 
 	// if the default is to save to cloud storage, pick something that will work as local file:
 	// simply extract the branch name which should be the users email address
-	if (default_filename && strstr(default_filename, prefs.cloud_git_url)) {
+	if (default_filename && QString(default_filename).contains(QRegularExpression(CLOUD_HOST_PATTERN))) {
 		QString filename(default_filename);
-		filename.remove(prefs.cloud_git_url);
 		filename.remove(0, filename.indexOf("[") + 1);
 		filename.replace("]", ".ssrf");
 		default_filename = copy_qstring(filename);
@@ -1348,7 +1354,7 @@ QString MainWindow::displayedFilename(QString fullFilename)
 	QFileInfo fileInfo(f);
 	QString fileName(fileInfo.fileName());
 
-	if (fullFilename.contains(prefs.cloud_git_url)) {
+	if (fullFilename.contains(prefs.cloud_base_url)) {
 		QString email = fileName.left(fileName.indexOf('['));
 		return git_local_only ?
 			tr("[local cache for] %1").arg(email) :
@@ -1513,20 +1519,15 @@ void MainWindow::editCurrentDive()
 	if (mainTab->isEditing() || DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING)
 		return;
 
-	disableShortcuts();
-	DivePlannerPointsModel::instance()->clear();
+	disableShortcuts(false);
+	copy_dive(current_dive, &displayed_dive); // Work on a copy of the dive
 	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::ADD);
-	graphics->setAddState();
+	DivePlannerPointsModel::instance()->loadFromDive(&displayed_dive);
+	graphics->setAddState(&displayed_dive, 0);
 	setApplicationState(ApplicationState::EditDive);
-	DivePlannerPointsModel::instance()->loadFromDive(current_dive);
 	mainTab->enableEdition();
 }
 
-void MainWindow::turnOffNdlTts()
-{
-	qPrefTechnicalDetails::set_calcndltts(false);
-}
-
 void MainWindow::on_actionExport_triggered()
 {
 	DiveLogExportDialog diveLogExport;
