@@ -38,7 +38,7 @@ void Printer::putProfileImage(const QRect &profilePlaceholder, const QRect &view
 	int y = profilePlaceholder.y() - viewPort.y();
 	// use the placeHolder and the viewPort position to calculate the relative position of the dive profile.
 	QRect pos(x, y, profilePlaceholder.width(), profilePlaceholder.height());
-	profile->plotDive(dive, true, true);
+	profile->plotDive(dive, 0, true);
 
 	if (!printOptions.color_selected) {
 		QImage image(pos.width(), pos.height(), QImage::Format_ARGB32);
@@ -128,7 +128,6 @@ void Printer::render(int pages)
 	// keep original preferences
 	ProfileWidget2 *profile = MainWindow::instance()->graphics;
 	int profileFrameStyle = profile->frameStyle();
-	int animationOriginal = qPrefDisplay::animation_speed();
 	double fontScale = profile->getFontPrintScale();
 	double printFontScale = 1.0;
 
@@ -136,7 +135,6 @@ void Printer::render(int pages)
 	profile->setFrameStyle(QFrame::NoFrame);
 	profile->setPrintMode(true, !printOptions.color_selected);
 	profile->setToolTipVisibile(false);
-	qPrefDisplay::set_animation_speed(0);
 
 	// render the Qwebview
 	QPainter painter;
@@ -150,8 +148,12 @@ void Printer::render(int pages)
 
 	QSize originalSize = profile->size();
 	if (collection.count() > 0) {
-		printFontScale = (double)collection.at(0).geometry().size().height() / (double)profile->size().height();
-		profile->resize(collection.at(0).geometry().size());
+		// A "standard" profile has about 600 pixels in height.
+		// Scale the fonts in the printed profile accordingly.
+		// This is arbitrary, but it seems to work reasonably.
+		QSize size = collection[0].geometry().size();
+		printFontScale = size.height() / 600.0;
+		profile->resize(size);
 	}
 	profile->setFontPrintScale(printFontScale);
 
@@ -186,10 +188,9 @@ void Printer::render(int pages)
 	profile->setFontPrintScale(fontScale);
 	profile->setToolTipVisibile(true);
 	profile->resize(originalSize);
-	qPrefDisplay::set_animation_speed(animationOriginal);
 
 	//replot the dive after returning the settings
-	profile->plotDive(current_dive, true, true);
+	profile->plotDive(current_dive, dc_number, true);
 }
 
 //value: ranges from 0 : 100 and shows the progress of the templating engine
