@@ -13,7 +13,6 @@
 #include "qt-models/diveplannermodel.h"
 #include "desktop-widgets/divelistview.h"
 #include "core/selection.h"
-#include "profile-widget/profilewidget2.h"
 #include "desktop-widgets/diveplanner.h"
 #include "qt-models/divecomputerextradatamodel.h"
 #include "qt-models/divelocationmodel.h"
@@ -355,7 +354,7 @@ void MainTab::updateDiveInfo()
 {
 	ui.location->refreshDiveSiteCache();
 	// don't execute this while adding / planning a dive
-	if (editMode || MainWindow::instance()->graphics->isPlanner())
+	if (editMode || DivePlannerPointsModel::instance()->isPlanner())
 		return;
 
 	// If there is no current dive, disable all widgets except the last two,
@@ -504,19 +503,15 @@ void MainTab::acceptChanges()
 	ui.dateEdit->setEnabled(true);
 	hideMessage();
 
-	if (editMode) {
-		MainWindow::instance()->showProfile();
-		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::NOTHING);
-		Command::editProfile(&displayed_dive);
-	}
+	MainWindow::instance()->showProfile();
+	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::NOTHING);
+	Command::editProfile(&displayed_dive);
+
 	int scrolledBy = MainWindow::instance()->diveList->verticalScrollBar()->sliderPosition();
-	if (editMode) {
-		MainWindow::instance()->diveList->reload();
-		MainWindow::instance()->refreshDisplay();
-		MainWindow::instance()->refreshProfile();
-	} else {
-		MainWindow::instance()->refreshDisplay();
-	}
+	MainWindow::instance()->diveList->reload();
+	MainWindow::instance()->refreshDisplay();
+	MainWindow::instance()->refreshProfile();
+
 	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::NOTHING);
 	MainWindow::instance()->diveList->verticalScrollBar()->setSliderPosition(scrolledBy);
 	MainWindow::instance()->diveList->setFocus();
@@ -528,27 +523,22 @@ void MainTab::acceptChanges()
 
 void MainTab::rejectChanges()
 {
-	if (editMode && current_dive) {
-		if (QMessageBox::warning(MainWindow::instance(), TITLE_OR_TEXT(tr("Discard the changes?"),
-									       tr("You are about to discard your changes.")),
-					 QMessageBox::Discard | QMessageBox::Cancel, QMessageBox::Discard) != QMessageBox::Discard) {
-			return;
-		}
+	if (QMessageBox::warning(MainWindow::instance(), TITLE_OR_TEXT(tr("Discard the changes?"),
+								       tr("You are about to discard your changes.")),
+				 QMessageBox::Discard | QMessageBox::Cancel, QMessageBox::Discard) != QMessageBox::Discard) {
+		return;
 	}
+
 	ui.dateEdit->setEnabled(true);
 	editMode = false;
 	hideMessage();
 	// no harm done to call cancelPlan even if we were not PLAN mode...
 	DivePlannerPointsModel::instance()->cancelPlan();
 
-	// now make sure that the correct dive is displayed
-	if (current_dive)
-		copy_dive(current_dive, &displayed_dive);
-	else
-		clear_dive(&displayed_dive);
 	updateDiveInfo();
 
 	// show the profile and dive info
+	MainWindow::instance()->refreshDisplay();
 	MainWindow::instance()->refreshProfile();
 	MainWindow::instance()->setEnabledToolbar(true);
 	ui.editDiveSiteButton->setEnabled(!ui.location->text().isEmpty());
