@@ -11,10 +11,12 @@ struct event;
 class DiveEventItem : public DivePixmapItem {
 	Q_OBJECT
 public:
-	DiveEventItem(QGraphicsItem *parent = 0);
+	DiveEventItem(const struct dive *d, struct event *ev, struct gasmix lastgasmix,
+		      DivePlotDataModel *model, DiveCartesianAxis *hAxis, DiveCartesianAxis *vAxis,
+		      int speed, QGraphicsItem *parent = nullptr);
 	~DiveEventItem();
-	void setEvent(const struct dive *d, struct event *ev, struct gasmix lastgasmix);
-	struct event *getEvent();
+	const struct event *getEvent() const;
+	struct event *getEventMutable();
 	void eventVisibilityChanged(const QString &eventName, bool visible);
 	void setVerticalAxis(DiveCartesianAxis *axis, int speed);
 	void setHorizontalAxis(DiveCartesianAxis *axis);
@@ -31,7 +33,7 @@ private:
 	DiveCartesianAxis *vAxis;
 	DiveCartesianAxis *hAxis;
 	DivePlotDataModel *dataModel;
-	struct event *internalEvent;
+	struct event *ev;
 	const struct dive *dive;
 };
 
