@@ -38,6 +38,7 @@ class DiveRectItem;
 class DepthAxis;
 class DiveCartesianAxis;
 class DiveProfileItem;
+class DivePlannerPointsModel;
 class TimeAxis;
 class DiveTemperatureItem;
 class DiveHeartrateItem;
@@ -74,13 +75,16 @@ public:
 		COLUMNS
 	};
 
-	ProfileWidget2(QWidget *parent = 0);
+	// Pass null as plannerModel if no support for planning required
+	ProfileWidget2(DivePlannerPointsModel *plannerModel, QWidget *parent = 0);
 	~ProfileWidget2();
 	void resetZoom();
 	void scale(qreal sx, qreal sy);
-	void plotDive(const struct dive *d, bool force = false, bool clearPictures = false, bool instant = false);
+	void plotDive(const struct dive *d, int dc, bool clearPictures = false, bool instant = false);
+	void setProfileState(const struct dive *d, int dc);
+	void setPlanState(const struct dive *d, int dc);
+	void setAddState(const struct dive *d, int dc);
 	void setPrintMode(bool mode, bool grayscale = false);
-	bool getPrintMode() const;
 	bool isPointOutOfBoundaries(const QPointF &point) const;
 	bool isPlanner() const;
 	double getFontPrintScale() const;
@@ -94,9 +98,6 @@ public:
 
 signals:
 	void fontPrintScaleChanged(double scale);
-	void enableToolbar(bool enable);
-	void enableShortcuts();
-	void disableShortcuts(bool paste);
 	void editCurrentDive();
 
 public
@@ -105,22 +106,21 @@ slots: // Necessary to call from QAction's signals.
 	void actionRequestedReplot(bool triggered);
 	void divesChanged(const QVector<dive *> &dives, DiveField field);
 	void setEmptyState();
-	void setProfileState();
 #ifndef SUBSURFACE_MOBILE
 	void plotPictures();
 	void picturesRemoved(dive *d, QVector<QString> filenames);
 	void picturesAdded(dive *d, QVector<PictureObj> pics);
-	void setPlanState();
-	void setAddState();
+	void pointsReset();
 	void pointInserted(const QModelIndex &parent, int start, int end);
 	void pointsRemoved(const QModelIndex &, int start, int end);
+	void pointsMoved(const QModelIndex &, int start, int end, const QModelIndex &destination, int row);
 	void updateThumbnail(QString filename, QImage thumbnail, duration_t duration);
 	void profileChanged(dive *d);
 	void pictureOffsetChanged(dive *d, QString filename, offset_t offset);
 	void removePicture(const QString &fileUrl);
 
 	/* this is called for every move on the handlers. maybe we can speed up this a bit? */
-	void recreatePlannedDive();
+	void divePlannerHandlerMoved();
 
 	/* key press handlers */
 	void keyEscAction();
@@ -135,6 +135,8 @@ slots: // Necessary to call from QAction's signals.
 #endif
 
 private:
+	void setProfileState(); // keep currently displayed dive
+	void updateAxes(); // Update axes according to preferences
 	void resizeEvent(QResizeEvent *event) override;
 #ifndef SUBSURFACE_MOBILE
 	void wheelEvent(QWheelEvent *event) override;
@@ -154,7 +156,6 @@ private:
 	void scrollViewTo(const QPoint &pos);
 	void setupSceneAndFlags();
 	template<typename T, class... Args> T *createItem(const DiveCartesianAxis &vAxis, int vColumn, int z, Args&&... args);
-	void setupItemSizes();
 	void addItemsToScene();
 	void setupItemOnScene();
 	void disconnectTemporaryConnections();
@@ -177,6 +178,7 @@ private:
 	void splitCurrentDC();
 
 	DivePlotDataModel *dataModel;
+	DivePlannerPointsModel *plannerModel; // If null, no planning supported.
 	int zoomLevel;
 	qreal zoomFactor;
 	bool isGrayscale;
@@ -189,6 +191,8 @@ private:
 	// All those here should probably be merged into one structure,
 	// So it's esyer to replicate for more dives later.
 	// In the meantime, keep it here.
+	const struct dive *d;
+	int dc;
 	struct plot_info plotInfo;
 	DepthAxis *profileYAxis;
 	PartialGasPressureAxis *gasYAxis;
@@ -219,15 +223,13 @@ private:
 	DiveHeartrateItem *heartBeatItem;
 	DiveCartesianAxis *percentageAxis;
 	QList<DivePercentageItem *> allPercentages;
-	DiveAmbPressureItem *ambPressureItem;
-	DiveGFLineItem *gflineItem;
 	DiveLineItem *mouseFollowerVertical;
 	DiveLineItem *mouseFollowerHorizontal;
 	RulerItem2 *rulerItem;
 #endif
 	TankItem *tankItem;
 
-	QList<QGraphicsSimpleTextItem *> gases;
+	std::vector<std::unique_ptr<QGraphicsSimpleTextItem>> gases;
 
 	//specifics for ADD and PLAN
 #ifndef SUBSURFACE_MOBILE
@@ -251,10 +253,14 @@ private:
 	void updateThumbnailPaintOrder();
 #endif
 
-	QList<DiveHandler *> handles;
+	std::vector<std::unique_ptr<DiveHandler>> handles;
+	int handleIndex(const DiveHandler *h) const;
 #ifndef SUBSURFACE_MOBILE
+	void connectPlannerModel();
 	void repositionDiveHandlers();
 	int fixHandlerIndex(DiveHandler *activeHandler);
+	DiveHandler *createHandle();
+	QGraphicsSimpleTextItem *createGas();
 #endif
 	friend class DiveHandler;
 	QHash<Qt::Key, QAction *> actionsForKeys;
@@ -263,6 +269,12 @@ private:
 	int maxtime;
 	int maxdepth;
 	double fontPrintScale;
+
+	// We store a const pointer to the shown dive. However, the undo commands want
+	// (understandably) a non-const pointer. Since the profile has a context-menu
+	// with actions, it needs such a non-const pointer. This function turns the
+	// currently shown dive into such a pointer. Ugly, yes.
+	struct dive *mutable_dive() const;
 };
 
 #endif // PROFILEWIDGET2_H
