@@ -1,6 +1,5 @@
 #!/bin/bash
 
 wget https://github.com/Kitware/CMake/releases/download/v3.13.2/cmake-3.13.2-Linux-x86_64.sh
-wget http://download.qt.io/official_releases/qt/5.12/5.12.4/qt-opensource-linux-x64-5.12.4.run
 curl -O https://dbus.freedesktop.org/releases/dbus/dbus-1.12.16.tar.gz
-wget https://github.com/openssl/openssl/archive/OpenSSL_1_1_1.tar.gz
+wget https://github.com/openssl/openssl/archive/OpenSSL_1_1_1i.tar.gz
