@@ -20,7 +20,9 @@ git status | grep "Changes not staged for commit" 2>/dev/null && echo "tree not
 git status | grep "Changes to be committed" 2>/dev/null && echo "tree not clean" && exit 1
 
 # now remove the translations and remove access to the kirigami sources
+# and any old sources under tmp
 chmod 000 mobile-widgets/qml/kirigami
+chmod 000 tmp
 rm translations/subsurface_source.ts
 
 # enable creating the translation strings
@@ -51,6 +53,7 @@ git reset --hard
 
 # now enable access to kirigami again
 chmod 755 mobile-widgets/qml/kirigami
+chmod 755 tmp
 
 # this really depends on my filesystem layout
 # push sources to Transifex
